package com.halomobi.network.report.track;

import android.content.ContentValues;
import android.database.Cursor;
import android.database.sqlite.SQLiteDatabase;
import android.provider.BaseColumns;
import android.text.TextUtils;

import com.halomobi.common.ConstantPool;
import com.halomobi.network.listeners.TrackerTaskListener;
import com.halomobi.network.report.db.DatabaseManager;
import com.halomobi.utils.FileUtils;
import com.halomobi.utils.LogUtils;

import java.util.HashSet;
import java.util.Set;

public class ReportTrackerManager {

    public static final String TABLE = "trackers";
    // Columns
    public static final String COLUMN_TRACK_URL = "url";
    public static final String COLUMN_TRACK_TIME = "last_request_time";
    /**
     * 监播过期时间
     */
    public static long EXPIRATION_TIME = 60 * 60 * 24 * 1000;
    private static ReportTrackerManager mInstance;
    private DatabaseManager dManager;    // 数据库的帮助类对象

    private ReportTrackerManager() {
        dManager = DatabaseManager.getInstance();
    }

    public synchronized static ReportTrackerManager getInstance() {
        if (mInstance == null) {
            mInstance = new ReportTrackerManager();
        }
        return mInstance;
    }

    /**
     * 上报监播
     *
     * @param tracks 监播集合
     */
    public void reportTrack(Set<String> tracks) {
        if (tracks.isEmpty()) {
            LogUtils.w("tracks isEmpty");
            return;
        }
        new ReportTrackerProcessor(tracks, null, false).process();
    }

//    /**
//     * * 上报监播
//     *
//     * @param tracks    监播集合
//     * @param trackType 保留
//     */
//    public void reportTrack(int trackType, String... tracks) {
//        for (String track : tracks) {
//            new ReportTrackerProcessor(track).process();
//        }
//    }

    /**
     * 上报监播
     *
     * @param tracks 监播集合
     */
    public void reportTrack(int trackType, final TrackerTaskListener listener, Set<String> tracks) {
        //Banner 需要接收展示监播回调触发轮播机制。
        if (trackType == ConstantPool.TrackerType.DISPLAY) {
            new ReportTrackerProcessor(tracks, listener, false).process();
            return;
        }

        reportTrack(tracks);

    }

    /**
     * 上传历史失败监播
     */
    synchronized void reportTrackerCache() {
        new ReportTrackerProcessor(getTrackerCaches(), null, true).process();
    }

    /**
     * 读取缓存监播数据
     *
     * @return 监播缓存
     */
    private Set<String> getTrackerCaches() {
        Set<String> trackers = new HashSet<>();
        long validTime = System.currentTimeMillis() - EXPIRATION_TIME;
        Cursor cursor = null;
        synchronized (DatabaseManager.class) {
            try {
                SQLiteDatabase db = dManager.openDatabase();
                cursor = db.query(TABLE,
                        new String[]{BaseColumns._ID, COLUMN_TRACK_URL},
                        COLUMN_TRACK_TIME + " >= ?",
                        new String[]{validTime + ""},
                        null,
                        null,
                        COLUMN_TRACK_TIME);
                if (cursor.moveToFirst()) {
                    do {
                        trackers.add(cursor.getString(cursor.getColumnIndex(COLUMN_TRACK_URL)));
                    } while (cursor.moveToNext());
                }
                //删除所有监播（包含过期的）
                db.delete(TABLE, null, null);
            } catch (Exception e) {
                e.printStackTrace();
            } finally {
                FileUtils.close(cursor);
                dManager.closeDatabase();
            }
            return trackers;
        }

    }

    /**
     * 监播发送失败时，本地记录
     *
     * @param track 监播地址
     */
    void addTracker(String track) {
        synchronized (DatabaseManager.class) {//设置写锁
            SQLiteDatabase db = null;
            try {
                if (TextUtils.isEmpty(track)) {
                    LogUtils.w("监播为空");
                    return;
                }
                db = dManager.openDatabase();
                db.beginTransaction();
                LogUtils.d("track  : " + track);
                ContentValues values = new ContentValues();
                values.put(COLUMN_TRACK_URL, track);
                values.put(COLUMN_TRACK_TIME, System.currentTimeMillis());
                db.insert(TABLE, null, values);
                db.setTransactionSuccessful();
            } catch (Exception e) {
                e.printStackTrace();
            } finally {
                if (db != null) {
                    if (db.inTransaction()) {
                        db.endTransaction();
                    }
                    dManager.closeDatabase();
                }
            }
        }
    }

}
